"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; } function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }// src/index.ts
var _pluginsyntaxjsx = require('@babel/plugin-syntax-jsx'); var _pluginsyntaxjsx2 = _interopRequireDefault(_pluginsyntaxjsx);
var _parser = require('@babel/parser');

// src/transform.ts
var _compiler = require('@vue-jsx-vapor/compiler');
var _generator = require('@babel/generator'); var _generator2 = _interopRequireDefault(_generator);

var _sourcemapjs = require('source-map-js');
var _traverse = require('@babel/traverse'); var _traverse2 = _interopRequireDefault(_traverse);
var transformJSX = (path, state) => {
  const { parent, node } = path;
  if (!(_optionalChain([parent, 'optionalAccess', _2 => _2.type]) !== "JSXExpressionContainer" && !isJSXElement(parent) && !isConditionalExpression(path.parentPath))) {
    return;
  }
  let { code, vaporHelpers, preamble, map } = _compiler.compile.call(void 0, 
    " ".repeat(_optionalChain([node, 'access', _3 => _3.loc, 'optionalAccess', _4 => _4.start, 'access', _5 => _5.column]) || 0) + _generator2.default.call(void 0, node).code,
    {
      mode: "module",
      inline: true,
      isTS: _optionalChain([state, 'access', _6 => _6.filename, 'optionalAccess', _7 => _7.endsWith, 'call', _8 => _8("tsx")]),
      filename: state.filename,
      sourceMap: true,
      ..._optionalChain([state, 'access', _9 => _9.opts, 'optionalAccess', _10 => _10.compile])
    }
  );
  vaporHelpers.forEach((helper) => state.opts.importSet.add(helper));
  preamble = preamble.replaceAll(
    /(?<=const )t(?=(\d))/g,
    `_t${state.opts.preambleIndex}`
  );
  code = code.replaceAll(/(?<== )t(?=\d)/g, `_t${state.opts.preambleIndex}`).replaceAll("_ctx: any", "").replaceAll("$event: any", "$event");
  state.opts.preambleIndex++;
  for (const [, key, value] of preamble.matchAll(
    /const (_t\d+) = (_template\(.*\))/g
  )) {
    const result = state.opts.preambleMap.get(value);
    if (result) {
      code = code.replaceAll(key, result);
    } else {
      state.opts.preambleMap.set(value, key);
    }
  }
  for (const [, events] of preamble.matchAll(/_delegateEvents\((.*)\)/g)) {
    events.split(", ").forEach((event) => state.opts.delegateEventSet.add(event));
  }
  const ast = _parser.parse.call(void 0, code, {
    sourceFilename: state.filename
  });
  if (map) {
    const consumer = new (0, _sourcemapjs.SourceMapConsumer)(map);
    const line = (_nullishCoalesce(_optionalChain([node, 'access', _11 => _11.loc, 'optionalAccess', _12 => _12.start, 'access', _13 => _13.line]), () => ( 1))) - 1;
    _traverse2.default.call(void 0, ast, {
      Identifier({ node: id }) {
        const originalLoc = consumer.originalPositionFor(id.loc.start);
        if (originalLoc) {
          id.loc = {
            ...id.loc,
            start: {
              line: line + originalLoc.line,
              column: originalLoc.column,
              index: originalLoc.column
            },
            end: {
              line: line + originalLoc.line,
              column: originalLoc.column + id.name.length,
              index: originalLoc.column + id.name.length
            }
          };
        }
      }
    });
  }
  path.replaceWith(ast.program.body[0]);
};
function isJSXElement(node) {
  return !!node && (node.type === "JSXElement" || node.type === "JSXFragment");
}
function isConditionalExpression(path) {
  return !!(path && (_optionalChain([path, 'optionalAccess', _14 => _14.type]) === "LogicalExpression" || path.type === "ConditionalExpression") && (path.parent.type === "JSXExpressionContainer" || path.parent.type === "ConditionalExpression" && isConditionalExpression(path.parentPath)));
}

// src/index.ts
var src_default = () => {
  return {
    name: "Vue JSX Vapor",
    inherits: _pluginsyntaxjsx2.default,
    visitor: {
      JSXElement: {
        exit: transformJSX
      },
      JSXFragment: {
        exit: transformJSX
      },
      Program: {
        enter: (_, state) => {
          state.opts.importSet = /* @__PURE__ */ new Set();
          state.opts.delegateEventSet = /* @__PURE__ */ new Set();
          state.opts.preambleMap = /* @__PURE__ */ new Map();
          state.opts.preambleIndex = 0;
        },
        exit: (path, { opts: { delegateEventSet, importSet, preambleMap } }) => {
          const statements = [];
          if (delegateEventSet.size) {
            statements.unshift(
              `_delegateEvents(${Array.from(delegateEventSet).join(", ")});`
            );
          }
          if (preambleMap.size) {
            let preambleResult = "";
            for (const [value, key] of preambleMap) {
              preambleResult += `const ${key} = ${value}
`;
            }
            statements.unshift(preambleResult);
          }
          const helpers = ["setText", "createTextNode"].filter((helper) => {
            const result = importSet.has(helper);
            result && importSet.delete(helper);
            return result;
          });
          if (helpers.length) {
            statements.unshift(
              `import { ${helpers.map((i) => `${i} as _${i}`).join(", ")} } from 'unplugin-vue-jsx-vapor/helper.js';
`
            );
          }
          if (importSet.size) {
            const importResult = Array.from(importSet).map((i) => `${i} as _${i}`).join(", ");
            statements.unshift(`import { ${importResult} } from 'vue/vapor';
`);
          }
          path.node.body.unshift(
            ..._parser.parse.call(void 0, statements.join("\n"), { sourceType: "module" }).program.body
          );
        }
      }
    }
  };
};


exports.default = src_default;

module.exports = exports.default;
